import unittest
import random
import awspice

class ServiceEc2TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Service.EC2 ")

    #################################
    # -------- INSTANCES ---------- #
    #################################

    def test_get_instances(self):
        aws = awspice.connect('eu-west-1')
        instances = aws.service.ec2.get_instances()
        self.assertTrue(len(instances) > 15)

    def test_get_instance_by(self):
        aws = awspice.connect('eu-west-2')
        instance = aws.service.ec2.get_instance_by('id', 'i-541f08dc', regions=['eu-west-1','eu-west-2'])
        self.assertEquals(instance['State']['Name'], 'running')

    def test_get_instances_by(self):
        aws = awspice.connect('eu-west-1')
        instances = aws.service.ec2.get_instances_by('status', 'running', regions=['eu-west-1','eu-west-2'])
        for instance in instances:
            self.assertEquals(instance['State']['Name'], 'running')

    def test_set_tag(self):
        aws = awspice.connect('eu-west-1')
        rnd = random.choice('abcdefghij')
        results = aws.service.ec2.set_tag(['i-056811c5f5e5fff57'], 'Test', 'david_amrani' + rnd , regions=['eu-west-1', 'eu-central-1'])


    #################################
    # ---------- VOLUMES ---------- #
    #################################

    def test_get_volumes(self):
        aws = awspice.connect('eu-west-1')
        vols = aws.service.ec2.get_volumes()
        inst = aws.service.ec2.get_instances()
        self.assertTrue(len(vols) > len(inst))


    def test_get_volumes_by(self):
        aws = awspice.connect('eu-west-1')
        volumes = aws.service.ec2.get_volumes_by('status', 'available')
        self.assertEquals(volumes[0]['State'], 'available')


    def test_get_volume_by(self):
        aws = awspice.connect('eu-west-1')
        volume = aws.service.ec2.get_volume_by('status', 'in-use')
        self.assertTrue('i-' in volume['Attachments'][0]['InstanceId'])



    # #################################
    # ----------- ADDRESSES -----------
    # #################################

    def test_get_addresses(self):
        aws = awspice.connect('eu-west-1')
        addresses = aws.service.ec2.get_addresses(regions=['eu-west-1'])
        addresses = filter(lambda x: x.get('AssociationId') != None, addresses)
        self.assertTrue(len(addresses) > 20)

    def test_get_address_by(self):
        aws = awspice.connect('eu-west-1')
        address = aws.service.ec2.get_address_by('privateip', '172.22.1.221', regions=['eu-west-1'])
        self.assertTrue(address['PublicIp'], '52.214.252.60')



if __name__ == '__main__':
        unittest.main()



import unittest
import random
import awspice

class ServiceS3TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Service.S3 ")

    #################################
    # --------  BUCKETS  ---------- #
    #################################

    def test_get_public_buckets(self):
        aws = awspice.connect('eu-central-1')
        buckets = aws.service.s3.get_public_buckets()
        self.assertEquals(len(buckets), 1)


if __name__ == '__main__':
        unittest.main()



# -*- coding: utf-8 -*-
from base import AwsBase

class AcmService(AwsBase):
    '''
    Class belonging to the ACM certificate management service.
    '''

    def list_certificates(self, regions=[]):
        '''
        List all certificates

        Args:
            regions (lst): List of regions to list certificates

        Returns:
            List of certificates
        '''
        certificates = list()
        regions = self.parse_regions(regions)

        for region in regions:
            self.change_region(region['RegionName'])
            certificates.extend(self.inject_client_vars(self.client.list_certificates()['CertificateSummaryList']))

        return certificates

    def get_certificate_by(self, filter_key, filter_value, regions=[]):
        '''
        Get certificate filtering by domain

        Args:
            filter_key (str): Name of the field to be searched. (Domain)
            filter_value (str): Value for the previous field. (i.e.: google.es)
            regions (lst): List of regions where the certificate can be.

        Returns:
            Certificate matched to the filter entered.
        '''
        filters = ['domain']
        if filter_key not in filters:
            raise Exception('Invalid filter key. Allowed filters: ' + str(filters))

        certificates = self.list_certificates(regions=regions)
        certificate_arn = filter(lambda x: x['DomainName'] == filter_value , certificates)
        if certificate_arn:
            return self.get_certificate(arn=certificate_arn[0]['CertificateArn'], regions=[certificate_arn[0]['RegionName']])
        else:
            return None

    def get_certificate(self, arn, regions=[]):
        '''
        Get certificate using CertificateArn (Ceritificate Identifier)

        Args:
            arn (str): ARN of the certificate
            regions (lst): List of regions where the certificate can be.

        Returns:
            Certificate matched to the ARN entered.
        '''
        regions = self.parse_regions(regions)
        for region in regions:
            self.change_region(region['RegionName'])
            certificate = self.inject_client_vars([self.client.describe_certificate(CertificateArn=arn)['Certificate']])[0]
            if certificate: return certificate
        return None


    def __init__(self):
        AwsBase.__init__(self, 'acm')



# -*- coding: utf-8 -*-
from base import AwsBase

class RdsService(AwsBase):
    '''
    Class belonging to the Remote Database system service.
    '''

    def get_rdss(self):
        '''
        Get all RDS for a region

        Returns:
            List of arrays with all RDS instances
        '''
        return self.inject_client_vars(self.client.describe_db_instances()['DBInstances'])

    def __init__(self):
        AwsBase.__init__(self, 'rds')



# -*- coding: utf-8 -*-
from base import AwsBase
import dns.resolver


class ElbService(AwsBase):
    '''
    Class belonging to the Load Balancers service.
    '''

    loadbalancer_filters = {
        'domain': '',
        'tagname': '',
        'cname': '',
    }

    @classmethod
    def _get_cname_from_domain(cls, domain):
        '''
        Get CNAME from a domain

        Raises:
            dns.resolver.NXDOMAIN: DNS Name not registered.
            dns.resolver.NoAnswer: DNS Name not found.

        Return:
            str: String with DNS Canonical Name.
        '''
        try:
            cname = str(dns.resolver.query(domain, "CNAME")[0]).rstrip('.')
            if 'aws.com' not in cname: raise ValueError('Domain %s is not in AWS' % domain)
            return cname
        except (dns.resolver.NXDOMAIN):
            raise dns.resolver.NXDOMAIN("Couldn't find any records(NXDOMAIN)")
        except (dns.resolver.NoAnswer):
            raise dns.resolver.NoAnswer("Couldn't find any records (NoAnswer)")


    def get_loadbalancers(self, regions=[]):
        '''
        Get all Elastic Load Balancers for a region

        Args:
            regions (list): Regions where to look for this element

        Returns:
            LoadBalancers (list): List of dictionaries with the load balancers requested
        '''
        results = list()
        regions = self.parse_regions(regions=regions)
        for region in regions:
            self.change_region(region['RegionName'])

            elbs = self.client.describe_load_balancers()['LoadBalancerDescriptions']
            results.extend(self.inject_client_vars(elbs))

        return results


    def get_loadbalancer_by(self, filter_key, filter_value, regions=[]):
        '''
        Get a load balancer for a region that matches with filter

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (list): Regions where to look for this element

        Raises:
            dns.resolver.NXDOMAIN: DNS Name not registered.
            dns.resolver.NoAnswer: DNS Name not found.

        Return:
            LoadBalancer (dict): Dictionary with the load balancer requested
        '''
        self.validate_filters(filter_key, self.loadbalancer_filters)

        if filter_key == 'tagname':
            for region in self.parse_regions(regions=regions):
                self.change_region(region)
                elbs = self.client.describe_load_balancers(LoadBalancerNames=[filter_value])
                elbs = elbs['LoadBalancerDescriptions']

        elif filter_key == 'domain' or filter_key == 'cname':

            if filter_key == 'domain':
                cname = self._get_cname_from_domain(filter_value)
            else:
                cname = filter_value

            self.change_region(cname.split('.')[1])
            elbs = [elb for elb in self.get_loadbalancers() if elb['DNSName'].lower() == cname.lower()]

        if elbs:
            return elbs[0]
        return None


    def __init__(self):
        AwsBase.__init__(self, 'elb')



import unittest
import awspice

class ServiceTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Service Class")

    #################################
    # -------- INSTANCES ---------- #
    #################################

    def test_get_auth_config(self):
        aws = awspice.connect('eu-west-1', profile='default')
        self.assertEquals(aws.service.get_auth_config(), {'Authorization': {'Type':'Profile', 'Value': 'default'}})

if __name__ == '__main__':
        unittest.main()



import unittest
import awspice

class ModuleStatsTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Module.Stats")

    def test_get_stats_of_region(self):
        aws = awspice.connect('eu-west-1', 'qa')
        stats = aws.stats.get_stats(regions=['eu-west-1'])
        self.assertTrue(isinstance(stats['Users'],list))
        self.assertTrue(isinstance(stats['Buckets'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['Instances'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['Volumes'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['SecurityGroups'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['Databases'],list))

    def test_get_stats_all_regions(self):
        aws = awspice.connect('eu-west-1', 'qa')
        stats = aws.stats.get_stats(regions='eu-west-1')
        self.assertTrue(isinstance(stats['Users'],list))
        self.assertTrue(isinstance(stats['Buckets'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['Instances'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['Volumes'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['SecurityGroups'],list))
        self.assertTrue(isinstance(stats['Regions']['eu-west-1']['Databases'],list))

    def test_cost_saving(self):
        aws = awspice.connect('eu-west-1')
        costs = aws.stats.cost_saving(regions='eu-west-1')
        self.assertTrue(isinstance(costs['Regions']['eu-west-1']['Volumes'],list))
        self.assertTrue(isinstance(costs['Regions']['eu-west-1']['Addresses'],list))
        self.assertTrue(isinstance(costs['Regions']['eu-west-1']['LoadBalancers'],list))


if __name__ == '__main__':
        unittest.main()



# -*- coding: utf-8 -*-
from base import AwsBase

class IamService(AwsBase):
    '''
    Class belonging to the IAM Identity & Access management service.
    '''

    def get_users(self):
        '''
        List all users for an AWS account

        Returns:
            List of all users
        '''
        user_list = []

        paginator = self.client.get_paginator('list_users')

        for response in paginator.paginate():
            for user in response['Users']:
                user_list.append(user)

        return user_list

    def __init__(self):
        AwsBase.__init__(self, 'iam')



import unittest
import re
from os.path import expanduser
import awspice

class ServiceBaseTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Service.Base")


    def test_inject_client_vars(self):
        aws = awspice.connect('eu-west-1', 'qa')

        elements = [{'test': 1}, {'test': 2}]
        newelements = aws.service.ec2.inject_client_vars(elements)

        for el in newelements:
            self.assertEquals(el['RegionName'], 'eu-west-1')
            self.assertEquals(el['Authorization']['Type'], 'Profile')
            self.assertEquals(el['Authorization']['Value'], 'qa')




    ####################################
    # ~~~~~~~~~~~  PROFILES ~~~~~~~~~~ #
    ####################################

    def test_get_profiles(self):
        aws = awspice.connect('eu-west-1')
        profiles = sorted(aws.service.ec2.get_profiles())

        # GET profiles from file
        with open(expanduser("~") + "/.aws/credentials", 'r') as myfile:
            data = myfile.read().replace('\n', '')
        regex = re.compile(r'\[(?P<name>\w*)\]')
        credentials = sorted(regex.findall(data))

        self.assertEquals(profiles, credentials)

    def test_change_profile(self):
        aws = awspice.connect('eu-west-1')
        for profile in ['default', 'qa']:
            aws.service.ec2.change_profile(profile)
            profile = aws.service.get_auth_config()
            self.assertEquals(profile['Authorization']['Value'], profile['Authorization']['Value'])

    def test_parse_profiles_empty(self):
        aws = awspice.connect('eu-west-1', profile='qa')
        profiles = aws.service.ec2.parse_profiles([])
        self.assertEquals(profiles, ['qa'])

    def test_parse_profiles_list(self):
        aws = awspice.connect('eu-west-1', profile='qa')
        profiles = aws.service.ec2.parse_profiles(['qa', 'test'])
        self.assertEquals(profiles, ['qa', 'test'])

    def test_parse_profiles_string(self):
        aws = awspice.connect('eu-west-1', profile='qa')
        profiles = aws.service.ec2.parse_profiles('test_str')
        self.assertEquals(profiles, ['test_str'])

    def test_parse_profiles_string_ALL_hack(self):
        aws = awspice.connect('eu-west-1', profile='qa')
        profiles = sorted(aws.service.ec2.parse_profiles('ALL'))

        # GET profiles from file
        with open(expanduser("~") + "/.aws/credentials", 'r') as myfile:
            data = myfile.read().replace('\n', '')
        regex = re.compile(r'\[(?P<name>\w*)\]')
        credentials = sorted(regex.findall(data))

        self.assertEquals(profiles, credentials)




    ####################################
    # ~~~~~~~~~    REGIONS   ~~~~~~~~~ #
    ####################################

    def test_get_regions(self):
        aws = awspice.connect('eu-west-1')
        regions = [ 'us-east-1',
                    'us-east-2',
                    'us-west-1',
                    'us-west-2',
                    'eu-west-3',
                    'eu-west-1',
                    'eu-west-2',
                    'eu-central-1',
                    'ca-central-1',
                    'ap-south-1',
                    'ap-southeast-1',
                    'ap-southeast-2',
                    'ap-northeast-1',
                    'ap-northeast-2',
                    'sa-east-1']
        regions = sorted(regions)
        curregions = sorted([region['RegionName'] for region in aws.service.ec2.get_regions()])
        self.assertEquals(regions, curregions)

    def test_change_regions(self):
        aws = awspice.connect('eu-west-1')
        for region in ['eu-west-1','eu-west-2','eu-central-1']:
            aws.service.ec2.change_region(region)
            curRegion = aws.service.ec2.region
            self.assertEquals(curRegion, region)

    def test_parse_regions_empty(self):
        curRegion = 'eu-west-1'
        aws = awspice.connect(curRegion)
        results = sorted(aws.service.ec2.parse_regions([]))
        self.assertEquals(results[0]['RegionName'], aws.service.ec2.region)

    def test_parse_regions_empty_getALL(self):
        aws = awspice.connect('eu-west-1')
        regionsEmpty = []
        results = sorted(aws.service.ec2.parse_regions(regionsEmpty, True))
        allRegions = sorted(aws.service.ec2.get_regions())
        self.assertEquals(results, allRegions)

    def test_parse_regions_string(self):
        aws = awspice.connect('eu-west-1')
        regionsStr = 'eu-west-1'
        resultsStr = aws.service.ec2.parse_regions(regionsStr)
        self.assertEqual(resultsStr, [{'RegionName': 'eu-west-1'}])

    def test_parse_regions_list_strings(self):
        aws = awspice.connect('eu-west-1')
        regionsList = ['eu-west-1', 'eu-central-1']
        resultsList = aws.service.ec2.parse_regions(regionsList)
        self.assertIn({'RegionName':regionsList[0]}, resultsList)
        self.assertIn({'RegionName':regionsList[1]}, resultsList)



if __name__ == '__main__':
        unittest.main()



import unittest
import random
import awspice

class ServiceCeTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Service.CE")

    def test_get_cost(self):
        time_from = '2018-02-01'
        time_to = '2018-02-28'
        aws = awspice.connect('eu-central-1')
        instance = aws.service.ec2.get_instance_by('id', 'i-0ac0ed3329e4114ae')
        cost = aws.service.ce.get_cost([instance['TagName']], time_from, time_to)
        self.assertGreater(float(cost[0]['Total']['UnblendedCost']['Amount']), 8.67)


if __name__ == '__main__':
        unittest.main()



# -*- coding: utf-8 -*-
import json
import datetime
import socket
from time import mktime

class ClsEncoder(json.JSONEncoder):
    '''
    JSON encoder extension.

    Sometimes Boto3 returns a non-serializable result to JSON and we get the following error when dumping that result:
    `TypeError: datetime.datetime (2015, 12, 3, 21, 20, 17, 326000, tzinfo = tzutc ()) is not JSON serializable`
    Solve it using this class encoder in ``cls`` argument

    Examples:
        json.dumps(results, indent=4, cls=awspice.ClsEncoder)
    '''

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return int(mktime(obj.timetuple()))

        return json.JSONEncoder.default(self, obj)


def ip_in_aws(ip):
    '''
    Check if an IP address is from AWS

    Arguments:
        ip: Address to check 

    Returns:
        bool
    '''
    return bool(dnsinfo_from_ip(ip))

def dnsinfo_from_ip(ip):
    '''
    Returns the DNS name of an IP address

    Arguments:
        ip: Address of the element.

    Examples:
        dns = get_dnsname_from_ip('8.8.8.8')

    Returns:
        dict: {'region': 'eu-west-1', 'service': 'ec2'}
    '''
    result = {}
    service_matchs = {'compute': 'ec2'}
    try:
        hostname = socket.gethostbyaddr(ip)[0]
        if hostname.endswith('aws.com'):
            result['region'] = hostname.split('.')[1]
            service = hostname.split('.')[2]
            result['service'] = service_matchs[service] if service in service_matchs.keys() else service
    except socket.herror:
        pass
    return result



# -*- coding: utf-8 -*-
from awspice.services.base import AwsBase
from awspice.helpers import dnsinfo_from_ip
from botocore.exceptions import ClientError
import types

class Ec2Service(AwsBase):
    '''
    Class belonging to the EC2 Computing service.
    '''

    ami_filters = {
        'id': 'image-id',
        'name': 'name',
        'architecture': 'architecture',
        'platform': 'platform',
    }

    ami_distributions = {
        'ubuntu': 'ubuntu/images/hvm-ssd/ubuntu-*-*{version}*-amd64-server-*',
        'windows': 'Windows_Server-*{version}*-English-*-Base-20*.*.*',
        'amazon': 'amzn-ami-hvm-20*.*.*-x86_64-*',
    }

    instance_filters = {
        'id': 'instance-id',
        'dnsname': 'dns-name',
        'publicip': 'network-interface.association.public-ip',
        'privateip': 'private-ip-address',
        'name': 'tag:Name',
        'tagname': 'tag:Name',
        'status': 'instance-state-name',
        'user': 'key-name',
    }

    distrib_amis = {
        'ubuntu': 'ami-f90a4880',
        'windows': 'ami-b5530b5e',
        'redhat': 'ami-c86c3f23',
    }

    snapshot_filters = {
        'id': 'snapshot-id',
        'status': 'status',
    }

    volume_filters = {
        'id': 'volume-id',
        'status': 'status',
        'tagname': 'tag:Name',
    }

    secgroup_filters = {
        'id': 'group-id',
        'description': 'description',
    }

    address_filters = {
        'publicip': 'public-ip',
        'privateip': 'private-ip-address'
    }


    # #################################
    # -----------  COMMON  -----------
    # #################################

    def set_tag(self, resource_id, tag_key, tag_value, regions=[]):
        '''
        Set tag for an instance

        Args:
            elements_id (str): Id of resources to tag. (i.e: i-01234, vol-01234)
            tag_key (str): Name of the element TAG (i.e: Name)
            tag_value (str): Value of that Tag
            regions (lst): Regions where to look for this element

        Returns:
            None
        '''
        tags = [{'Key': tag_key, 'Value': tag_value}]
        curRegion = AwsBase.region
        regions = self.parse_regions(regions)

        for region in regions:
            self.change_region(region['RegionName'])
            try:
                self.client.create_tags(Resources=resource_id, Tags=tags)
                return None
            except ClientError:
                pass
            finally:
                self.change_region(curRegion)

        raise Exception('The element {} does not exist for that regions.'.format(resource_id))

    # #################################
    # -------------- AMIS -------------
    # #################################

    def _extract_amis(self, filters=[], regions=[], return_first=False):
        filters.append({'Name': 'state', 'Values': ['available', 'pending']})
        # Just supported x64 OS
        filters.append({'Name': 'architecture', 'Values': ['x86_64']})
        filters.append({'Name': 'hypervisor', 'Values': ['xen']})
        filters.append({'Name': 'virtualization-type', 'Values': ['hvm']})
        filters.append({'Name': 'image-type', 'Values': ['machine']})
        filters.append({'Name': 'root-device-type', 'Values': ['ebs']})

        curRegion = AwsBase.region
        regions = self.parse_regions(regions)
        results = list()

        for region in regions:
            self.change_region(region['RegionName'])

            amis = self.client.describe_images(Filters=filters)['Images']
            amis = self.inject_client_vars(amis)
            if return_first and amis:
                self.change_region(curRegion)
                return amis[0]
            results.extend(amis)

        self.change_region(curRegion)
        return results

    def get_amis_by_distribution(self, distrib, version='*', latest=False, regions=[]):
        '''
        Get one or more Images filtering by distribution

        Args:
            distrib (str): Distribution of the image (i.e.: ubuntu)
            version (str): Version of the system
            latest (bool): True if only returns the newest item.

        Return:
            Image (lst): List with the images requested.
        '''

        self.validate_filters(distrib, self.ami_distributions.keys())
        filters = [
            {'Name': 'name', 'Values': [self.ami_distributions[distrib].format(version=version)]},
            {'Name': 'is-public', 'Values': ['true']}
        ]

        results = self._extract_amis(filters=filters, regions=regions)
        results = sorted(results, key=lambda k: k['Name'])

        if latest and results:
            return [results[-1]]

        return results

    def get_ami_by(self, filter_key, filter_value, regions=[]):
        '''
        Get an ami for one or more regions that matches with filter

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (lst): Regions where to look for this element

        Return:
            Image (dict): Image requested
        '''
        return self.get_amis_by(filter_key=filter_key,
                                filter_value=filter_value,
                                regions=regions,
                                return_first=True)

    def get_amis_by(self, filter_key, filter_value, regions=[], return_first=False):
        '''
        Get list of amis for one or more regions that matches with filter

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (lst): Regions where to look for this element
            return_first (bool): True if return first result

        Return:
            Images (lst): List of requested images
        '''

        self.validate_filters(filter_key, self.ami_filters)

        filters = [{
            'Name': self.ami_filters[filter_key],
            'Values': [filter_value]
        }]

        return self._extract_amis(filters=filters, regions=regions, return_first=return_first)

    def get_amis(self, regions=[]):
        '''
        Get all images

        Args:
            regions (lst): Regions where to look for this element

        Returns:
            Images (lst): List of all images
        '''
        return self._extract_amis(regions=regions)


    # #################################
    # ----------- INSTANCES -----------
    # #################################

    def _extract_instances(self, filters=[], regions=[], return_first=False):
        results = list()
        curRegion = AwsBase.region
        regions = self.parse_regions(regions)

        for region in regions:
            self.change_region(region['RegionName'])

            reservations = self.client.describe_instances(Filters=filters)["Reservations"]
            for reserv in reservations:
                instances = self.inject_client_vars(reserv['Instances'])

                if return_first and instances:
                    self.change_region(curRegion)
                    return instances[0]

                results.extend(instances)

        self.change_region(curRegion)
        return results

    def get_instances(self, regions=[]):
        '''
        Get all instances for one or more regions.

        Args:
            regions (lst): Regions where to look for this element

        Returns:
            Instances (lst): List of dictionaries with the instances requested
        '''
        return self._extract_instances(regions=regions)

    def get_instance_by(self, filter_key, filter_value, regions=[]):
        '''
        Get an instance for one or more regions that matches with filter

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (lst): Regions where to look for this element

        Return:
            Instance (dict): Dictionary with the instance requested
        '''
        return self.get_instances_by(filter_key, filter_value, regions, return_first=True)

    def get_instances_by(self, filter_key, filter_value, regions=[], return_first=False):
        '''
        Get an instance for one or more regions that matches with filter

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (lst): Regions where to look for this element
            return_first (bool): Select to return the first match

        Return:
            Instances (lst): List of dictionaries with the instances requested
        '''
        self.validate_filters(filter_key, self.instance_filters)

        filters = [{
            'Name': self.instance_filters[filter_key],
            'Values': [filter_value]
        }]

        # DNS name of AWS instances ends with region.service.amazonaws.com.
        if (filter_key == 'publicip'):
            if (dnsinfo_from_ip(filter_value)):
                regions = [dnsinfo_from_ip(filter_value)['region']]
            else:
                return list()

        return self._extract_instances(filters=filters, regions=regions, return_first=return_first)

    def create_instances(self, name, key_name, allowed_range, ami=None, distribution=None,
                         version=None, instance_type='t2.micro', region=None, vpc=None, count=1):
        '''
        Create a new instance

        Args:
            name (str): TagName of the instance
            key_name (str): The name of the key pair (i.e: it_user)
            allowed_range (str): Network range with access to instance (i.e: 10.0.0.0/32)
            ami (str): Id of the ami (i.e: ami-12345)
            instance_type (str): Type of hardware of the instance (i.e: t2.medium)
            distribution (str): Instead of ami, select an OS: (i.e: ubuntu)
            region (str): Name of the region where  instance will be displayed
            vpc (str): VPC identifier where the instance will be deployed.
            count (int): Number of instances to launch

        Returns:
            Instances (lst): List of launched instances
        '''

        if region:
            curRegion = AwsBase.region
            self.change_region(region)

        if not vpc:
            vpc = self.get_default_vpc()['VpcId']

        if not ami:
            latest_ami = []
            if distribution and version:
                latest_ami = self.get_amis_by_distribution(distribution, version, latest=True)
            elif distribution and not version:
                latest_ami = self.get_amis_by_distribution(distribution, latest=True)

            if latest_ami:
                ami = latest_ami[0]['ImageId']
            else:
                raise ValueError("Insert a valid AMI or distribution.\n" +
                                 "Parameters: Distribution={distrib}; Version={version}; ami={ami}".format(
                        distrib=distribution,
                        version=version,
                        ami=ami))

        secgroup_id = str()
        try:
            secgroup_id = self.create_security_group(name, allowed_range, vpc)
            instance = self.resource.create_instances(
                ImageId=ami,
                InstanceType=instance_type,
                KeyName=key_name,
                SecurityGroupIds=[secgroup_id],
                MaxCount=count,
                MinCount=count,
                TagSpecifications=[
                    {'ResourceType': 'instance', 'Tags': [
                        {'Key': 'Name', 'Value': name}]}
                ]
            )
            return instance
        except Exception:
            if secgroup_id:
                self.delete_security_group(secgroup_id)
            raise
        finally:
            self.change_region(curRegion)


    def start_instances(self, instance_ids, regions=[]):
        '''
        Stops an Amazon EC2 instance

        Args:
            instance_ids (lst): List of identifiers of instances to be started.

        Examples:
            $ aws.service.ec2.start_instances(instances=['i-001'])
            $ aws.service.ec2.start_instances(instances=['i-001', 'i-033'], regions=['eu-west-1', 'eu-central-1'])

        Returns:
            lst: List of instances to be started, with their previous and current status.
        '''
        regions = self.parse_regions(regions)
        started_instances = list()

        for region in regions:
            self.change_region(region['RegionName'])

            for instance in instance_ids:
                try:
                    x = self.client.start_instances(InstanceIds=[instance])
                    started_instances.extend(x['StartingInstances'])
                    instance_ids.remove(instance)

                except ClientError:
                    pass

        return started_instances


    def stop_instances(self, instance_ids, regions=[], force=False):
        '''
        Stops an Amazon EC2 instance

        Args:
            instance_ids (lst): List of identifiers of instances to be stopped.

        Examples:
            $ aws.service.ec2.stop_instances(instances=['i-001'])
            $ aws.service.ec2.stop_instances(instances=['i-001', 'i-033'], regions=['eu-west-1', 'eu-central-1'])

        Returns:
            lst: List of instances to be stopped, with their previous and current status.
        '''
        regions = self.parse_regions(regions)
        stopped_instances = list()

        for region in regions:
            self.change_region(region['RegionName'])

            for instance in instance_ids:
                try:
                    x = self.client.stop_instances(InstanceIds=[instance], Force=force)
                    stopped_instances.extend(x['StoppingInstances'])
                    instance_ids.remove(instance)

                except ClientError:
                    pass

        return stopped_instances



    # #################################
    # ------------ VOLUMES ------------
    # #################################

    def _extract_volumes(self, filters=[], regions=[], return_first=False):
        results = list()

        curRegion = AwsBase.region
        regions = self.parse_regions(regions)
        for region in regions:
            self.change_region(region['RegionName'])

            volumes = self.client.describe_volumes(Filters=filters)['Volumes']
            volumes = self.inject_client_vars(volumes)

            if return_first and volumes:
                self.change_region(curRegion)
                return volumes[0]

            results.extend(volumes)

        self.change_region(curRegion)
        return results

    def get_volumes(self, regions=[]):
        '''
        Get all volumes for one or more regions

        Args:
            regions (lst): Regions where to look for this element

        Returns:
            Volumes (lst): List of dictionaries with the volumes requested
        '''
        return self._extract_volumes(regions=regions)

    def get_volume_by(self, filter_key, filter_value, regions=[]):
        '''
        Get a volume for one or more regions that matches with filters

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (lst): Regions where to look for this element

        Returns:
            Volume (dict): Dictionary with the volume requested
        '''
        return self.get_volumes_by(filter_key, filter_value, regions, return_first=True)

    def get_volumes_by(self, filter_key, filter_value, regions=[], return_first=False):
        '''
        Get volumes for one or more regions that matches with filters

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter
            regions (lst): Regions where to look for this element

        Returns:
            Volume (dict): Dictionary with the volume requested
        '''
        self.validate_filters(filter_key, self.instance_filters)

        filters = [{
            'Name': self.volume_filters[filter_key],
            'Values': [filter_value]
        }]

        return self._extract_volumes(filters=filters, regions=regions, return_first=return_first)



    # #################################
    # ----------- SNAPSHOTS -----------
    # #################################

    def get_snapshots(self):
        '''
        Get all snapshots owned by self for the current region

        Returns:
            Snapshots (lst): List of dictionaries with the snapshots requested
        '''
        snapshots = self.client.describe_snapshots(OwnerIds=['self'])['Snapshots']
        return self.inject_client_vars(snapshots)

    def get_snapshot_by(self, filter_key, filter_value):
        '''
        Get a snapshot for a region tha matches with filters

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter

        Returns:
            Snapshot (dict): Dictionary with the snapshot requested
        '''
        snapshots = self.get_snapshots_by(filter_key, filter_value)
        if snapshots and snapshots:
            return self.inject_client_vars(snapshots)[0]
        return None

    def get_snapshots_by(self, filter_key, filter_value):
        '''
        Get all snapshots for the current region that matches with filters

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter

        Returns:
            Snapshots (lst): List of dictionaries with the snapshots requested
        '''
        self.validate_filters(filter_key, self.snapshot_filters)

        filters = [{
            'Name': self.snapshot_filters[filter_key],
            'Values': [filter_value]
        }]
        return self.inject_client_vars(self.client.describe_snapshots(Filters=filters)['Snapshots'])



    # #################################
    # ---------- SEC. GROUPS ----------
    # #################################

    def get_secgroups(self):
        '''
        Get all security groups for the current region

        Returns:
            SecurityGroups (lst): List of dictionaries with the security groups requested
        '''
        return self.inject_client_vars(self.client.describe_security_groups()['SecurityGroups'])

    def get_secgroup_by(self, filter_key, filter_value):
        '''
        Get security group for a region that matches with filters

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter

        Returns:
            SecurityGroup (dict): Dictionaries with the security group requested
        '''
        secgroup = self.get_secgroups_by(filter_key, filter_value)
        if secgroup:
            return self.inject_client_vars(secgroup)[0]
        return None

    def get_secgroups_by(self, filter_key, filter_value):
        '''
        Get all security groups for a region that matches with filters

        Args:
            filter_key (str): Name of the filter
            filter_value (str): Value of the filter

        Returns:
            SecurityGroups (lst): List of dictionaries with the security groups requested
        '''
        self.validate_filters(filter_key, self.secgroup_filters)
        filters = [{
            'Name': self.secgroup_filters[filter_key],
            'Values': [filter_value]
        }]

        secgroups = self.client.describe_security_groups(Filters=filters)['SecurityGroups']
        return self.inject_client_vars(secgroups)

    def create_security_group(self, name, allowed_range, vpc_id=None):
        '''
        Create a new Security Group

        Args:
            name (str): Name of the Security Group
            allowed_range (str): Network range with permissions (i.e: 10.0.0.0/32)
            vpc_id (str): Id of assigned VPC

        Returns:
            str: Identifier of the security group created.
        '''
        vpc = vpc_id if vpc_id else self.get_default_vpc()['VpcId']
        desc = 'Security group created by Awspice'
        default_rules = [
            {'IpProtocol': 'tcp',
             'FromPort': 80,
             'ToPort': 80,
             'IpRanges': [{'CidrIp': allowed_range}]},
            {'IpProtocol': 'tcp',
             'FromPort': 22,
             'ToPort': 22,
             'IpRanges': [{'CidrIp': allowed_range}]}
        ]

        sg_id = self.client.create_security_group(
            GroupName=name, VpcId=vpc, Description=desc)['GroupId']
        self.client.authorize_security_group_ingress(
            GroupId=sg_id, IpPermissions=default_rules)

        return sg_id

    def delete_security_group(self, identifier):
        '''
        Delete an existing Security Group

        Args:
            identifier (str): Id of the Security Group

        Returns:
            none
        '''
        self.client.delete_security_group(GroupId=identifier)

    # #################################
    # ----------- ADDRESSES -----------
    # #################################

    def _extract_addresses(self, filters=[], regions=[], return_first=False):
        results = list()

        curRegion = AwsBase.region
        regions = self.parse_regions(regions)
        for region in regions:
            self.change_region(region['RegionName'])

            addresses = self.client.describe_addresses(Filters=filters)['Addresses']
            addresses = self.inject_client_vars(addresses)

            if return_first and addresses:
                self.change_region(curRegion)
                return addresses[0]

            results.extend(addresses)

        self.change_region(curRegion)
        return results

    def get_addresses(self, regions=[]):
        '''
        Get all IP Addresses for a region

        Args:
            regions (lst): Regions where to look for this element

        Returns:
            Addresses (dict): List of dictionaries with the addresses requested
        '''
        return self._extract_addresses(regions=regions)

    def get_address_by(self, filter_key, filter_value, regions=[]):
        '''
        Get IP Addresses for a region that matches with filters

        Args:
            regions (lst): Regions where to look for this element

        Returns:
            Address (dict): Dictionary with the address requested
        '''
        self.validate_filters(filter_key, self.address_filters)
        filters = [{
            'Name': self.address_filters[filter_key],
            'Values': [filter_value]
        }]
        return self._extract_addresses(filters=filters, regions=regions, return_first=True)



    # #################################
    # ------------- VPCS --------------
    # #################################

    def _extract_vpcs(self, filters=[], regions=[], return_first=False):
        results = list()

        curRegion = AwsBase.region
        regions = self.parse_regions(regions)
        for region in regions:
            self.change_region(region['RegionName'])

            vpcs = self.client.describe_vpcs(Filters=filters)['Vpcs']
            vpcs = self.inject_client_vars(vpcs)

            if return_first and vpcs:
                self.change_region(curRegion)
                return vpcs[0]

            results.extend(vpcs)

        self.change_region(curRegion)
        return results

    def get_vpcs(self, regions=[]):
        '''
        Get all VPCs for a region

        Returns:
            VPCs (lst): List of dictionaries with the vpcs requested
        '''
        return self._extract_vpcs(regions=regions)

    def get_default_vpc(self):
        '''
        Get default Security Group

        Returns:
            SecurityGroup (dict): Default security group resource
        '''
        vpcs = self.get_vpcs()

        vpc = filter(lambda x: x['IsDefault'] is True, vpcs)
        return vpc[0]



    def __init__(self):
        AwsBase.__init__(self, 'ec2')



import unittest
import awspice

class ModuleFinderTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Module.Find_Elements")


    def test_find_instance_accounts(self):
        aws = awspice.connect('eu-west-2')
        instance = aws.finder.find_instance('id', 'i-541f08dc', profiles=['qa', 'default'])
        self.assertEquals(instance['InstanceId'], 'i-541f08dc')

    def test_find_instance_regions(self):
        aws = awspice.connect('eu-west-2', profile='default')
        instance = aws.finder.find_instance('id', 'i-541f08dc',regions=['eu-west-1'])
        self.assertEquals(instance['InstanceId'], 'i-541f08dc')

    def test_find_instances(self):
        aws = awspice.connect('eu-west-1')
        instances = aws.finder.find_instances('status', 'running', profiles=['qa'])
        self.assertEquals(instances[0]['State']['Name'], 'running')
        # Verify regions=[] == all regions
        q_regions = map(lambda x: x['RegionName'], instances)
        map(lambda x: self.assertTrue(x in ['eu-central-1', 'eu-west-1']), q_regions)


    def test_find_volume(self):
        aws = awspice.connect('eu-west-1')
        volume = aws.finder.find_volume('id', 'vol-04ee612f6b83104cd', profiles=['default', 'qa'])
        self.assertEquals(volume['State'], 'in-use')

    def test_find_volumes(self):
        aws = awspice.connect('eu-west-1')
        volumes = aws.finder.find_volumes('status', 'in-use', profiles=['qa'])
        self.assertEquals(volumes[2]['State'], 'in-use')



if __name__ == '__main__':
        unittest.main()



import unittest
import random
import awspice

class ServiceElbTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("\nStarting unit tests of Service.ELB ")

    #################################
    # -------- BALANCERS ---------- #
    #################################

    def test_get_loadbalancers(self):
        aws = awspice.connect('eu-central-1')
        elbs = aws.service.elb.get_loadbalancers()
        self.assertTrue(len(elbs) ==20)

    def test_get_loadbalancer_by(self):
        aws = awspice.connect('eu-central-1')
        elb = aws.service.elb.get_loadbalancer_by('domain', 'aws.elevenpaths.com')
        self.assertEquals(elb['CanonicalHostedZoneName'], 'elevenpaths-web-pro-1763591843.eu-west-1.elb.amazonaws.com')


if __name__ == '__main__':
        unittest.main()



from acm import AcmService
from base import AwsBase
from ce import CostExplorerService
from ec2 import Ec2Service
from elb import ElbService
from iam import IamService
from rds import RdsService
from s3 import S3Service

__all__ = ['AwsBase', 'Ec2Service', 'ElbService', 'IamService', 'RdsService', 'S3Service', 'AcmService', 'CostExplorerService']



# -*- coding: utf-8 -*-
from base import AwsBase
import datetime

class CostExplorerService(AwsBase):
    '''
    Class belonging to the Cost Explorer service.
    '''

    def get_cost(self, tag_names, from_date=None, to_date=None, interval="Monthly"):
        '''
        Get the cost of item/s by its tag "Name".

        This method obtains the price of one or several elements (substances, balancers, addresses)
        between two dates and granularized in days or months.
        If the date is not indicated, the cost of the last month will be returned.

        Args:
            tagnames (lst): List of tag names of machines to obtain data (wildcards are not valid)
            from_date (str): Date from which you want to obtain data. (Format: 2018-04-24)
            to_date (str): Date until which you want to obtain data. (Format: 2018-04-24)
            interval (str):Time interval to be analyzed. [MONTHLY|DAILY]

        Examples:
            get_cost(['machine-1', 'machine-2'], '2018-12-24', '2018-12-26', interval='daily')

        Returns:
            Costs (list): List of days or months with the requested costs
        '''

        # If the dates are empty, data from the last month will be extracted.
        if from_date is None and to_date is None:
            from_date = (datetime.datetime.now() -
                datetime.timedelta(1*365/12)).strftime('%Y-%m-%d')
            to_date = datetime.datetime.now().strftime('%Y-%m-%d')

        # Parameters validation
        try:
            datetime.datetime.strptime(from_date, '%Y-%m-%d')
            datetime.datetime.strptime(to_date, '%Y-%m-%d')
            if not interval.upper() in ["DAILY", "MONTHLY"]:
                raise AssertionError()
        except:
            raise ValueError("Invalid parameters")

        timeperiod = {'Start': from_date , 'End': to_date}
        filters = {"Tags": {"Key": "Name", "Values": tag_names}}

        results = self.client.get_cost_and_usage(TimePeriod=timeperiod,
                                                 Granularity=interval.upper(),
                                                 Filter=filters,
                                                 Metrics=['UnblendedCost'])
        return results['ResultsByTime']


    def __init__(self):
        AwsBase.__init__(self, 'ce')
        self.change_region('us-east-1')



# -*- coding: utf-8 -*-
from awspice.helpers import ip_in_aws

class FinderModule:
    '''
    This class makes it easy to search for components in AWS.

    Attributes:
        aws: awspice client

    '''


    def find_instance(self, filter_key, filter_value, profiles=[], regions=[]):
        '''
        Searches for an instance in different accounts and regions, using search filters.
        '''
        profiles = self.aws.ec2.parse_profiles(profiles)
        regions = self.aws.ec2.parse_regions(regions, True)

        if filter_key == 'publicip' and not ip_in_aws(filter_value):
            return None

        for account in profiles:
            self.aws.ec2.change_profile(account)
            instance = self.aws.ec2.get_instance_by(filter_key, filter_value, regions=regions)
            if instance: return instance
        return None

    def find_instances(self, filter_key=None, filter_value=None, profiles=[], regions=[]):
        '''
        Searches for a group of instances in different accounts and regions, using search filters.
        '''
        results = list()
        profiles = self.aws.ec2.parse_profiles(profiles)
        regions = self.aws.ec2.parse_regions(regions, True)

        if filter_key == 'publicip' and not ip_in_aws(filter_value):
            return None

        for account in profiles:
            self.aws.ec2.change_profile(account)
            if filter_key and filter_value:
                results.extend(self.aws.ec2.get_instances_by(filter_key, filter_value, regions=regions))
            else:
                results.extend(self.aws.ec2.get_instances(regions))
        return results

    def find_volume(self, filter_key, filter_value, profiles=[], regions=[]):
        '''
        Searches for an volume in different accounts and regions, using search filters.
        '''
        profiles = self.aws.ec2.parse_profiles(profiles)
        regions = self.aws.ec2.parse_regions(regions, True)

        for account in profiles:
            self.aws.ec2.change_profile(account)
            regions = self.aws.ec2.parse_regions(regions, True)
            volume = self.aws.ec2.get_volume_by(filter_key, filter_value, regions=regions)
            if volume: return volume
        return None

    def find_volumes(self, filter_key, filter_value, profiles=[], regions=[]):
        '''
        Searches for a group of volumes in different accounts and regions, using search filters.
        '''
        results = list()
        profiles = self.aws.ec2.parse_profiles(profiles)
        regions = self.aws.ec2.parse_regions(regions, True)

        for account in profiles:
            self.aws.ec2.change_profile(account)
            regions = self.aws.ec2.parse_regions(regions, True)
            results.extend(self.aws.ec2.get_volumes_by(filter_key, filter_value, regions=regions))
        return results


    def __init__(self, aws):
        self.aws = aws



# -*- coding: utf-8 -*-
from awspice.services.base import AwsBase
from botocore.exceptions import ClientError
import StringIO

class S3Service(AwsBase):
    '''
    Class belonging to the S3 Storage service.
    '''

    def upload_string_as_file(self, bucket_name, filepath, content):
        '''
        Upload string as a file to S3 bucket

        Args:
            bucket_name (str): Name of the S3 bucket
            filepath (str): File path which will be created. (i.e. 'folder1/folder2/filename.txt')
            content (str): File content in string format.

        Returns:
            None
        '''
        data = StringIO.StringIO(content)
        self.resource.Bucket(bucket_name).put_object(Key=filepath, Body=data)


    def get_buckets(self):
        '''
        Get all buckets in S3

        Returns:
            Buckets (list): List of dictionaries with the buckets requested
        '''
        return self.inject_client_vars(self.client.list_buckets()['Buckets'])

    def get_public_buckets(self):
        '''
        Get all public readable or writable buckets in S3

        Returns:
            Buckets-ACL (list): List of dictionaries with the buckets requested
        '''
        buckets = []
        for bucket in self.get_buckets():

            try:
                bucket_acl = self.client.get_bucket_acl(Bucket=bucket['Name'])

                # https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html
                global_acl = 'http://acs.amazonaws.com/groups/global/AllUsers'
                permissions = ['READ', 'WRITE']
                public_bucket = {}

                for grant in bucket_acl['Grants']:
                    if grant.get('Permission') in permissions and grant['Grantee'].get('URI') == global_acl:
                        if public_bucket.get('Name'):
                            public_bucket['Permissions'].append(grant['Permission'])
                        else:
                            public_bucket['Name'] = bucket['Name']
                            public_bucket['Permissions'] = []
                            public_bucket['Permissions'].append(grant['Permission'])

                if public_bucket:
                    buckets.append(public_bucket)

            except ClientError:
                # Exception: Lack of permissions
                pass

        return self.inject_client_vars(buckets)

    def __init__(self):
        AwsBase.__init__(self, 's3')
